@@ -361,7 +361,7 @@ end_gnutls_io (GTlsConnectionGnutls  *gnutls,
         {
           g_clear_error (&my_error);
           g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS,
-                       _("Peer failed to perform TLS handshake: %s"), gnutls_strerror (status));
+                       _("Peer failed to perform TLS handshake: %s"), gnutls_strerror (ret));
           return G_TLS_CONNECTION_BASE_ERROR;
         }
     }
@@ -385,7 +385,7 @@ end_gnutls_io (GTlsConnectionGnutls  *gnutls,
         {
           g_clear_error (&my_error);
           g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_NOT_TLS,
-                       _("Peer failed to perform TLS handshake: %s"), gnutls_strerror (status));
+                       _("Peer failed to perform TLS handshake: %s"), gnutls_strerror (ret));
           return G_TLS_CONNECTION_BASE_ERROR;
         }
 
@@ -1020,6 +1020,12 @@ g_tls_connection_gnutls_write (GTlsConnectionBase  *tls,
   ret = gnutls_record_send (priv->session, buffer, count);
   END_GNUTLS_IO (gnutls, G_IO_OUT, ret, status, _("Error writing data to TLS socket"), error);
 
+  /* GnuTLS will resend interrupted writes unless we discard its send queue.
+   * But GOutputStream requires explicit resend when interrupted.
+   */
+  if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN)
+    gnutls_record_discard_queued (priv->session);
+
   *nwrote = MAX (ret, 0);
   return status;
 }
